@@ -7,13 +7,19 @@ import proxy_server, json
 
 def expose_service(methods, public=False):
     def decorator(view_func):
+        view_func.__dict__['_proxy_service'] = True
+
         def wrapper(request, *args, **kwargs):
             error_message = None
             try:
                 if hasattr(settings, 'PROXY_API_KEYS'):
                     if request.META.get(proxy_server.HTTP_API_KEY) in settings.PROXY_API_KEYS:
                         if hasattr(settings, 'PROXY_TOKEN_VALIDATION_SERVICE'):
-                            if public is False and request.META.get(proxy_server.HTTP_USER_TOKEN):
+                            if public is True and request.META.get(proxy_server.HTTP_USER_TOKEN) is not None:
+                                error_message = 'A public service cannot receive a user token'
+                                raise Exception
+
+                            elif public is False and request.META.get(proxy_server.HTTP_USER_TOKEN):
                                 try:
                                     dot = settings.PROXY_TOKEN_VALIDATION_SERVICE.rindex('.')
                                 except ValueError:
@@ -26,7 +32,6 @@ def expose_service(methods, public=False):
                                 try:
                                     response = val_func(request)
                                 except Exception as e:
-                                    print 'ERROR: ',e
                                     error_message = 'Could not invoke token validation service'
                                     raise Exception
 
